import {
  select,
  selectAll,
  csv,
  scaleLinear,
  extent,
  axisLeft,
  axisBottom,
  scaleOrdinal,
  schemeDark2
} from 'd3';

const svg = select('svg');

const width = +svg.attr('width');
const height = +svg.attr('height');

const render = data => {
  
  const title = 'Cars: Horsepower vs Weight';
  
  const xValue = d => d.horsepower;
  const xAxisLabel = 'Horsepower';
  
  const yValue = d => d.weight;
  const circleRadius = 6;
  const yAxisLabel = 'Weight';
  
  // create color value
  const origin = [...new Set(data.map(d => d.origin))];
  const colorValue = d => d.origin;
  const colorLabel = 'Origin';
  const colorScale = scaleOrdinal()
    .domain(origin)
    .range(schemeDark2);
  
  const margin = { top: 60, right: 40, bottom: 88, left: 150 };
  const innerWidth = width - margin.left - margin.right;
  const innerHeight = height - margin.top - margin.bottom;
  
  const xScale = scaleLinear()
    .domain(extent(data, xValue))
    .range([0, innerWidth])
    .nice();
  
  const yScale = scaleLinear()
    .domain(extent(data, yValue))
    .range([innerHeight, 0])
    .nice();
  
  const g = svg.append('g')
    .attr('transform', `translate(${margin.left},${margin.top})`);
  
  const xAxis = axisBottom(xScale)
    .tickSize(-innerHeight)
    .tickPadding(15);
  
  const yAxis = axisLeft(yScale)
    .tickSize(-innerWidth)
    .tickPadding(10);
  
  const yAxisG = g.append('g').call(yAxis);
  yAxisG.selectAll('.domain').remove();
  
  yAxisG.append('text')
      .attr('class', 'axis-label')
      .attr('y', -93)
      .attr('x', -innerHeight / 2)
      .attr('fill', 'black')
      .attr('transform', `rotate(-90)`)
      .attr('text-anchor', 'middle')
      .text(yAxisLabel);
  
  const xAxisG = g.append('g').call(xAxis)
    .attr('transform', `translate(0,${innerHeight})`);
  
  xAxisG.select('.domain').remove();
  
  xAxisG.append('text')
      .attr('class', 'axis-label')
      .attr('y', 75)
      .attr('x', innerWidth / 2)
      .attr('fill', 'black')
      .text(xAxisLabel);
  
  // create legend group
  const colorLegendG = g.selectAll('.legend')
    .data(colorScale.domain())
    .enter().append('g')
      .attr('class', 'legend')
      .attr('transform', `translate(30, 25)`);
  
  // create legend circles
  colorLegendG.append('circle')
    .attr('class', 'legend-circle')
    .attr('r', 10)
    .attr('fill', colorScale)
    .attr('transform', (d, i) =>
        `translate(0, ${i * 25})`);

  // create legend text
  colorLegendG.append('text')
    .attr('class', 'legend-text')
    .attr('transform', (d, i) =>
        `translate(0, ${i * 25})`
    )
    .attr('x', circleRadius * 3)
    .attr('dy', '0.32em')
    .text(d => d);
  
  // create the click event
  let selected;
  colorLegendG.on('click', (type, i, nodes) => {
    
    selectAll('.data-circle').attr('opacity', 1);
	selectAll('.legend').attr('opacity', 1);
    
    if (selected !== type) {
      selectAll('.legend').attr('opacity', 0.1);
      select(nodes[i]).attr('opacity', 1);
      selectAll('.data-circle')
        .filter(d => {
          return d.origin !== type;})
          .attr('opacity', 0.1);
      selected = type;
    }
    else {selected = null;}
  });
  
  g.selectAll('circle').data(data)
    .enter().append('circle')
      .attr('class', 'data-circle')
      .attr('cy', d => yScale(yValue(d)))
      .attr('cx', d => xScale(xValue(d)))
      .attr('r', circleRadius)
      // color differentiate dots by origin
      .attr('fill', d => colorScale(colorValue(d)));
  
  g.append('text')
      .attr('class', 'title')
      .attr('y', -10)
      .text(title);
};

csv('https://vizhub.com/curran/datasets/auto-mpg.csv')
  .then(data => {
    data.forEach(d => {
      d.mpg = +d.mpg;
      d.cylinders = +d.cylinders;
      d.displacement = +d.displacement;
      d.horsepower = +d.horsepower;
      d.weight = +d.weight;
      d.acceleration = +d.acceleration;
      d.year = +d.year;
      
    });
    render(data);
  });